/**
 * @file
 * Common code for store tests
 *
 * @authors
 * Copyright (C) 2020 Richard Russon <rich@flatcap.org>
 *
 * @copyright
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TEST_STORE_COMMON_H
#define TEST_STORE_COMMON_H

#include "acutest.h"
#include <stdbool.h>

struct Buffer;
struct StoreOps;

bool test_store_db(const struct StoreOps *sops, void *db);
bool test_store_degenerate(const struct StoreOps *sops, const char *name);
bool test_store_setup(struct Buffer *path);

#endif /* TEST_STORE_COMMON_H */
